@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
-# Copyright 2011 Martin Manns
+# Copyright Martin Manns
 # Distributed under the terms of the GNU General Public License
 
 # --------------------------------------------------------------------
@@ -53,11 +53,11 @@ class TestCellActions(object):
         self.grid = self.main_window.grid
         self.code_array = self.grid.code_array
 
-    param_set_code = [ \
-       {'key': (0, 0, 0), 'code': "'Test'", 'result': "'Test'"},
-       {'key': (0, 0, 0), 'code': "", 'result': None},
-       {'key': (0, 0, 1), 'code': None, 'result': None},
-       {'key': (999, 99, 2), 'code': "4", 'result': "4"},
+    param_set_code = [
+        {'key': (0, 0, 0), 'code': "'Test'", 'result': "'Test'"},
+        {'key': (0, 0, 0), 'code': "", 'result': None},
+        {'key': (0, 0, 1), 'code': None, 'result': None},
+        {'key': (999, 99, 2), 'code': "4", 'result': "4"},
     ]
 
     @params(param_set_code)
@@ -70,13 +70,6 @@ class TestCellActions(object):
 
         assert self.grid.code_array(key) == result
 
-        wx.Yield()
-
-        if code is None or not code:
-            assert not self.main_window.changed_since_save
-        else:
-            assert self.main_window.changed_since_save
-
     @params(param_set_code)
     def test_delete_cell(self, key, code, result):
         """Unit test for delete_cell"""
@@ -86,17 +79,17 @@ class TestCellActions(object):
 
         assert self.grid.code_array(key) is None
 
-    param_get_reference = [ \
-       {'cursor': (0, 0, 0), 'ref_key': (0, 0, 0), 'abs_ref': "S[0, 0, 0]",
-        'rel_ref': "S[X, Y, Z]"},
-       {'cursor': (0, 0, 1), 'ref_key': (0, 0, 1), 'abs_ref': "S[0, 0, 1]",
-        'rel_ref': "S[X, Y, Z]"},
-       {'cursor': (0, 0, 0), 'ref_key': (0, 0, 1), 'abs_ref': "S[0, 0, 1]",
-        'rel_ref': "S[X, Y, Z+1]"},
-       {'cursor': (9, 0, 0), 'ref_key': (0, 0, 0), 'abs_ref': "S[0, 0, 0]",
-        'rel_ref': "S[X-9, Y, Z]"},
-       {'cursor': (23, 2, 1), 'ref_key': (2, 2, 2), 'abs_ref': "S[2, 2, 2]",
-        'rel_ref': "S[X-21, Y, Z+1]"},
+    param_get_reference = [
+        {'cursor': (0, 0, 0), 'ref_key': (0, 0, 0), 'abs_ref': "S[0, 0, 0]",
+         'rel_ref': "S[X, Y, Z]"},
+        {'cursor': (0, 0, 1), 'ref_key': (0, 0, 1), 'abs_ref': "S[0, 0, 1]",
+         'rel_ref': "S[X, Y, Z]"},
+        {'cursor': (0, 0, 0), 'ref_key': (0, 0, 1), 'abs_ref': "S[0, 0, 1]",
+         'rel_ref': "S[X, Y, Z+1]"},
+        {'cursor': (9, 0, 0), 'ref_key': (0, 0, 0), 'abs_ref': "S[0, 0, 0]",
+         'rel_ref': "S[X-9, Y, Z]"},
+        {'cursor': (23, 2, 1), 'ref_key': (2, 2, 2), 'abs_ref': "S[2, 2, 2]",
+         'rel_ref': "S[X-21, Y, Z+1]"},
     ]
 
     @params(param_get_reference)
@@ -119,19 +112,21 @@ class TestCellActions(object):
     def test_append_reference_code(self, cursor, ref_key, abs_ref, rel_ref):
         """Unit test for append_reference_code"""
 
+        actions = self.grid.actions
+
         params = [
             # Normal initial code, absolute reference
             {'initial_code': "3 + ", 'ref_type': "absolute",
-             "res": self.grid.actions._get_absolute_reference(ref_key)},
+             "res": actions._get_absolute_reference(ref_key)},
             # Normal initial code, relative reference
             {'initial_code': "3 + ", 'ref_type': "relative",
-             "res": self.grid.actions._get_relative_reference(cursor, ref_key)},
+             "res": actions._get_relative_reference(cursor, ref_key)},
             # Initial code with reference, absolute reference
             {'initial_code': "3 + S[2, 3, 1]", 'ref_type': "absolute",
-             "res": self.grid.actions._get_absolute_reference(ref_key)},
+             "res": actions._get_absolute_reference(ref_key)},
             # Initial code with reference, relative reference
             {'initial_code': "3 + S[2, 3, 1]", 'ref_type': "relative",
-             "res": self.grid.actions._get_relative_reference(cursor, ref_key)},
+             "res": actions._get_relative_reference(cursor, ref_key)},
         ]
 
         for param in params:
@@ -141,8 +136,8 @@ class TestCellActions(object):
 
             self.grid.actions.set_code(cursor, initial_code)
 
-            result_code = self.grid.actions.append_reference_code(cursor,
-                                                          ref_key, ref_type)
+            result_code = \
+                actions.append_reference_code(cursor, ref_key, ref_type)
 
             if "S[" in initial_code:
                 assert result_code == initial_code[:4] + res
@@ -150,7 +145,7 @@ class TestCellActions(object):
             else:
                 assert result_code == initial_code + res
 
-    param_set_cell_attr = [ \
+    param_set_cell_attr = [
         {'selection': Selection([], [], [], [], [(2, 5)]), 'tab': 1,
          'attr': ('bordercolor_right', wx.RED), 'testcell': (2, 5, 1)},
         {'selection': Selection([(0, 0)], [(99, 99)], [], [], []), 'tab': 0,
@@ -177,9 +172,6 @@ class TestCellActions(object):
 
         assert color == attr[attr.keys()[0]]
 
-        wx.Yield()
-        assert self.main_window.changed_since_save
-
     def test_set_border_attr(self):
         """Unit test for set_border_attr"""
 
@@ -188,7 +180,7 @@ class TestCellActions(object):
         attr = "borderwidth"
         value = 5
         borders = ["top", "inner"]
-        tests = { \
+        tests = {
             (13, 14, 0): 5,
             (53, 14, 0): 1,
         }
@@ -213,11 +205,11 @@ class TestCellActions(object):
             res = self.grid.code_array.cell_attributes[cell]["underline"]
             assert res == tests[cell]
 
-    param_change_frozen_attr = [ \
+    param_change_frozen_attr = [
         {'cell': (0, 0, 0), 'code': None, 'result': None},
         {'cell': (0, 0, 0), 'code': "'Test'", 'result': 'Test'},
-        {'cell': (2, 1, 1), 'code': "'Test'", 'result': 'Test'},
-        {'cell': (2, 1, 1), 'code': "32", 'result': 32},
+        {'cell': (2, 2, 0), 'code': "'Test'", 'result': 'Test'},
+        {'cell': (2, 1, 0), 'code': "32", 'result': 32},
     ]
 
     @params(param_change_frozen_attr)
@@ -238,14 +230,14 @@ class TestCellActions(object):
 
         res2 = self.grid.code_array.cell_attributes[cell]["frozen"]
 
-        assert res2 == False
+        assert not res2
 
-    param_get_new_cell_attr_state = [ \
+    param_get_new_cell_attr_state = [
         {'cell': (0, 0, 0), 'attr': "fontweight",
          'before': wx.NORMAL, 'next': wx.BOLD},
-        {'cell': (2, 1, 3), 'attr': "fontweight",
+        {'cell': (2, 1, 0), 'attr': "fontweight",
          'before': wx.NORMAL, 'next': wx.BOLD},
-        {'cell': (2, 1, 3), 'attr': "vertical_align",
+        {'cell': (2, 1, 0), 'attr': "vertical_align",
          'before': "top", 'next': "middle"},
     ]
 
@@ -263,17 +255,17 @@ class TestCellActions(object):
 
         assert res == next
 
-    param_get_new_selection_attr_state = [ \
+    param_get_new_selection_attr_state = [
         {'selection': Selection([], [], [], [], [(0, 0)]), 'cell': (0, 0, 0),
          'attr': "fontweight", 'before': wx.NORMAL, 'next': wx.BOLD},
-        {'selection': Selection([], [], [], [], [(2, 1)]), 'cell': (2, 1, 2),
+        {'selection': Selection([], [], [], [], [(2, 1)]), 'cell': (2, 1, 0),
          'attr': "fontweight", 'before': wx.NORMAL, 'next': wx.BOLD},
-        {'selection': Selection([], [], [], [], [(2, 1)]), 'cell': (2, 1, 2),
+        {'selection': Selection([], [], [], [], [(2, 1)]), 'cell': (2, 1, 0),
          'attr': "fontweight", 'before': wx.BOLD, 'next': wx.NORMAL},
-        {'selection': Selection([], [], [], [], [(2, 1)]), 'cell': (2, 1, 2),
+        {'selection': Selection([], [], [], [], [(2, 1)]), 'cell': (2, 1, 0),
          'attr': "vertical_align", 'before': "top", 'next': "middle"},
         {'selection': Selection([(1, 0)], [(23, 2)], [], [], []),
-         'cell': (2, 1, 2),
+         'cell': (2, 1, 0),
          'attr': "vertical_align", 'before': "top", 'next': "middle"},
     ]
 
@@ -321,4 +313,4 @@ class TestCellActions(object):
         # Refresh cell
         selection = Selection([], [], [], [], [cell[:2]])
         self.grid.actions.refresh_selected_frozen_cells(selection=selection)
-        assert self.grid.code_array[cell] == 2
\ No newline at end of file
+        assert self.grid.code_array[cell] == 2
