@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
-# Copyright 2011 Martin Manns
+# Copyright Martin Manns
 # Distributed under the terms of the GNU General Public License
 
 # --------------------------------------------------------------------
@@ -29,15 +29,20 @@ from ast import literal_eval
 
 import wx
 
-from sysvars import get_color, get_font_string
+##from sysvars import get_color, get_font_string
 
-VERSION = "0.2.3"
+VERSION = "0.2.4"
 
 
 class DefaultConfig(object):
     """Contains default config for starting pyspread without resource file"""
 
     def __init__(self):
+        # Config file version
+        # -------------------
+
+        self.config_version = VERSION
+
         # User defined paths
         # ------------------
 
@@ -60,8 +65,6 @@ class DefaultConfig(object):
         # Grid configuration
         # ------------------
 
-        ##self.grid_shape = "(1000, 100, 3)"
-
         self.grid_rows = "1000"
         self.grid_columns = "100"
         self.grid_tables = "3"
@@ -72,14 +75,14 @@ class DefaultConfig(object):
         self.max_result_length = "1000"
 
         # Colors
-        self.grid_color = repr(get_color(wx.SYS_COLOUR_3DSHADOW))
-        self.selection_color = repr(get_color(wx.SYS_COLOUR_HIGHLIGHT))
-        self.background_color = repr(get_color(wx.SYS_COLOUR_WINDOW))
-        self.text_color = repr(get_color(wx.SYS_COLOUR_WINDOWTEXT))
+        self.grid_color = repr(wx.SYS_COLOUR_GRAYTEXT)
+        self.selection_color = repr(wx.SYS_COLOUR_HIGHLIGHT)
+        self.background_color = repr(wx.SYS_COLOUR_WINDOW)
+        self.text_color = repr(wx.SYS_COLOUR_WINDOWTEXT)
 
         # Fonts
 
-        self.font = repr(get_font_string(wx.SYS_DEFAULT_GUI_FONT))
+        self.font = repr(wx.SYS_DEFAULT_GUI_FONT)
 
         # Default cell font size
 
@@ -129,6 +132,9 @@ class Config(object):
 
         self.cfg_file = wx.Config(self.config_filename)
 
+        # Config keys to be resetted to default value on version upgrades
+        self.reset_on_version_change = ["window_layout"]
+
         self.load()
 
     def __getitem__(self, key):
@@ -147,6 +153,9 @@ class Config(object):
     def load(self):
         """Loads configuration file"""
 
+        # Config files prior to 0.2.4 dor not have config version keys
+        old_config = not self.cfg_file.Exists("config_version")
+
         # Reset data
         self.data.__dict__.update(self.defaults.__dict__)
 
@@ -154,16 +163,25 @@ class Config(object):
             if self.cfg_file.Exists(key):
                 setattr(self.data, key, self.cfg_file.Read(key))
 
+        # Reset keys that should be reset on version upgrades
+        if old_config or self.version != self.data.config_version:
+            for key in self.reset_on_version_change:
+                setattr(self.data, key, getattr(DefaultConfig(), key))
+            self.data.config_version = self.version
+
+        # Reset GPG key if version prior 0.2.4
+        # because of pyme -> gnupg migration
+
+        if old_config:
+            key = "gpg_key_uid"
+            setattr(self.data, key, getattr(DefaultConfig(), key))
+
     def save(self):
         """Saves configuration file"""
 
         for key in self.defaults.__dict__:
             data = getattr(self.data, key)
 
-            if key == "gpg_key_passphrase" and \
-               not self["gpg_key_passphrase_isstored"]:
-                data = "''"
-
             self.cfg_file.Write(key, data)
 
-config = Config()
\ No newline at end of file
+config = Config()
