@@ -30,6 +30,7 @@ Provides:
 """
 
 import csv
+from itertools import islice
 import os
 import sys
 import types
@@ -41,7 +42,7 @@ import src.lib.i18n as i18n
 
 from _dialogs import MacroDialog, DimensionsEntryDialog, AboutDialog
 from _dialogs import CsvImportDialog, CellEntryDialog, CsvExportDialog
-from _dialogs import PreferencesDialog, GPGParamsDialog
+from _dialogs import PreferencesDialog, GPGParamsDialog, PasteAsDialog
 
 #use ugettext instead of getttext to avoid unicode errors
 _ = i18n.language.ugettext
@@ -99,7 +100,8 @@ class ModalDialogInterfaceMixin(object):
 
         msg = _("There are unsaved changes.\nDo you want to save?")
 
-        dlg = GMD.GenericMessageDialog(self.main_window, msg,
+        dlg = GMD.GenericMessageDialog(
+            self.main_window, msg,
             _("Unsaved changes"), wx.YES_NO | wx.ICON_QUESTION | wx.CANCEL)
 
         save_choice = dlg.ShowModal()
@@ -139,7 +141,7 @@ class ModalDialogInterfaceMixin(object):
         return filepath, filter_index
 
     def display_warning(self, message, short_message,
-                              style=wx.OK | wx.ICON_WARNING):
+                        style=wx.OK | wx.ICON_WARNING):
         """Displays a warning message"""
 
         dlg = GMD.GenericMessageDialog(self.main_window, message,
@@ -148,7 +150,7 @@ class ModalDialogInterfaceMixin(object):
         dlg.Destroy()
 
     def get_warning_choice(self, message, short_message,
-                        style=wx.YES_NO | wx.NO_DEFAULT | wx.ICON_WARNING):
+                           style=wx.YES_NO | wx.NO_DEFAULT | wx.ICON_WARNING):
         """Launches proceeding dialog and returns True if ok to proceed"""
 
         dlg = GMD.GenericMessageDialog(self.main_window, message,
@@ -199,8 +201,9 @@ class ModalDialogInterfaceMixin(object):
         except csv.Error, err:
             # Display modal warning dialog
 
-            msg = _("'{}' does not seem to be a valid CSV file.\n \nOpening it"
-                    " yielded the error:\n{}").format(csvfilename, err)
+            msg = _("'{filepath}' does not seem to be a valid CSV file.\n \n"
+                    "Opening it yielded the error:\n{error}")
+            msg = msg.format(filepath=csvfilename, error=err)
             short_msg = _('Error reading CSV file')
 
             self.display_warning(msg, short_msg)
@@ -220,14 +223,14 @@ class ModalDialogInterfaceMixin(object):
 
         return dialect, has_header, digest_types
 
-    def get_csv_export_info(self, data):
+    def get_csv_export_info(self, preview_data):
         """Shows csv export preview dialog and returns csv_info
 
         csv_info is a tuple of dialect, has_header, digest_types
 
         Parameters
         ----------
-        data: Iterable of iterables
+        preview_data: Iterable of iterables
         \tContains csv export data row-wise
 
         """
@@ -235,7 +238,9 @@ class ModalDialogInterfaceMixin(object):
         preview_rows = 100
         preview_cols = 100
 
-        export_preview = data[:preview_rows, :preview_cols]
+        export_preview = list(list(islice(col, None, preview_cols))
+                              for col in islice(preview_data, None,
+                                                preview_rows))
 
         filterdlg = CsvExportDialog(self.main_window, data=export_preview)
 
@@ -287,6 +292,22 @@ class ModalDialogInterfaceMixin(object):
 
         return integer
 
+    def get_pasteas_parameters_from_user(self, obj):
+        """Opens a PasteAsDialog and returns parameters dict"""
+
+        dlg = PasteAsDialog(None, -1, obj)
+        dlg_choice = dlg.ShowModal()
+
+        if dlg_choice != wx.ID_OK:
+            dlg.Destroy()
+            return None
+
+        parameters = {}
+        parameters.update(dlg.parameters)
+        dlg.Destroy()
+
+        return parameters
+
 
 class DialogInterfaceMixin(object):
     """Main window interfaces to dialogs that are not modal"""
