@@ -33,14 +33,17 @@ import ast
 import os
 
 import wx
-import wx.aui
+import wx.lib.agw.aui as aui
+
+from matplotlib.figure import Figure
 
 import src.lib.i18n as i18n
 from src.config import config
+from src.sysvars import get_python_tutorial_path, is_gtk
 
 from _menubars import MainMenu
 from _toolbars import MainToolbar, MacroToolbar, FindToolbar, AttributesToolbar
-from _widgets import EntryLine, StatusBar, TableChoiceIntCtrl
+from _widgets import EntryLineToolbarPanel, StatusBar
 
 from src.lib.clipboard import Clipboard
 
@@ -61,11 +64,17 @@ class MainWindow(wx.Frame, EventMixin):
     """Main window of pyspread"""
 
     def __init__(self, parent, *args, **kwargs):
+        try:
+            S = kwargs.pop("S")
+
+        except KeyError:
+            S = None
+
         wx.Frame.__init__(self, parent, *args, **kwargs)
 
         self.interfaces = GuiInterfaces(self)
 
-        self._mgr = wx.aui.AuiManager(self)
+        self._mgr = aui.AuiManager(self)
 
         self.parent = parent
 
@@ -101,7 +110,7 @@ class MainWindow(wx.Frame, EventMixin):
         self.attributes_toolbar = AttributesToolbar(self, -1)
 
         # Entry line
-        self.entry_line = EntryLine(self, style=wx.TE_PROCESS_ENTER)
+        self.entry_line_panel = EntryLineToolbarPanel(self, -1)
 
         # Main grid
 
@@ -111,10 +120,7 @@ class MainWindow(wx.Frame, EventMixin):
             config["grid_tables"],
         )
 
-        self.grid = Grid(self, -1, dimensions=dimensions)
-
-        # IntCtrl for table choice
-        self.table_choice = TableChoiceIntCtrl(self, dimensions[2])
+        self.grid = Grid(self, -1, S=S, dimensions=dimensions)
 
         # Clipboard
         self.clipboard = Clipboard()
@@ -164,10 +170,9 @@ class MainWindow(wx.Frame, EventMixin):
             (self.main_toolbar, "main_window_toolbar", _("Main toolbar")),
             (self.macro_toolbar, "macro_toolbar", _("Macro toolbar")),
             (self.attributes_toolbar, "attributes_toolbar",
-                                                       _("Format toolbar")),
+             _("Format toolbar")),
             (self.find_toolbar, "find_toolbar", _("Find toolbar")),
-            (self.entry_line, "entry_line", _("Entry line")),
-            (self.table_choice, "table_choice", _("Table choice")),
+            (self.entry_line_panel, "entry_line_panel", _("Entry line")),
         ]
 
         for toolbar, pane_name, toggle_label in toggles:
@@ -183,41 +188,33 @@ class MainWindow(wx.Frame, EventMixin):
             toggle_item.Check(pane.IsShown())
 
     def _do_layout(self):
-        """Adds widgets to the wx.aui manager and controls the layout"""
+        """Adds widgets to the aui manager and controls the layout"""
 
         # Add the toolbars to the manager
 
-        self._mgr.AddPane(self.macro_toolbar, wx.aui.AuiPaneInfo().
-            Name("macro_toolbar").Caption("Macro Toolbar").
-            ToolbarPane().Top().Row(0).CloseButton(False).
-            LeftDockable(False).RightDockable(False))
+        self._mgr.AddPane(self.main_toolbar, aui.AuiPaneInfo().
+                          Name("main_window_toolbar").
+                          Caption(_("Main toolbar")).
+                          ToolbarPane().Top().Row(0))
 
-        self._mgr.AddPane(self.main_toolbar, wx.aui.AuiPaneInfo().
-            Name("main_window_toolbar").Caption("Main Toolbar").
-            ToolbarPane().Top().Row(0).CloseButton(False).
-            LeftDockable(False).RightDockable(False))
+        self._mgr.AddPane(self.find_toolbar, aui.AuiPaneInfo().
+                          Name("find_toolbar").Caption(_("Find toolbar")).
+                          ToolbarPane().Top().Row(0))
 
-        self._mgr.AddPane(self.find_toolbar, wx.aui.AuiPaneInfo().
-            Name("find_toolbar").Caption("Find").
-            ToolbarPane().Top().Row(1).MaximizeButton(False).
-            LeftDockable(False).RightDockable(False))
+        self._mgr.AddPane(self.attributes_toolbar, aui.AuiPaneInfo().
+                          Name("attributes_toolbar").
+                          Caption(_("Format toolbar")).
+                          ToolbarPane().Top().Row(1))
 
-        self._mgr.AddPane(self.attributes_toolbar, wx.aui.AuiPaneInfo().
-            Name("attributes_toolbar").Caption("Cell Attributes").
-            ToolbarPane().Top().Row(1).MaximizeButton(False).
-            LeftDockable(False).RightDockable(False))
+        self._mgr.AddPane(self.macro_toolbar, aui.AuiPaneInfo().
+                          Name("macro_toolbar").Caption(_("Macro toolbar")).
+                          Gripper(True).ToolbarPane().Top().Row(1))
 
-        self._mgr.AddPane(self.table_choice, wx.aui.AuiPaneInfo().
-            Name("table_choice").Caption("Table choice").
-            ToolbarPane().MaxSize((50, 50)).Row(2).
-            Top().CloseButton(False).MaximizeButton(False).
-            LeftDockable(True).RightDockable(True))
-
-        self._mgr.AddPane(self.entry_line, wx.aui.AuiPaneInfo().
-            Name("entry_line").Caption("Entry line").
-            ToolbarPane().MinSize((10, 10)).Row(2).
-            Top().CloseButton(False).MaximizeButton(False).
-            LeftDockable(True).RightDockable(True))
+        self._mgr.AddPane(self.entry_line_panel, aui.AuiPaneInfo().
+                          Name("entry_line_panel").Caption(_("Entry line")).
+                          MinSize((10, 20)).Row(2).CaptionVisible(False).
+                          Top().CloseButton(True).Gripper(True).
+                          MaximizeButton(True))
 
         # Load perspective from config
         window_layout = config["window_layout"]
@@ -226,10 +223,13 @@ class MainWindow(wx.Frame, EventMixin):
             self._mgr.LoadPerspective(window_layout)
 
         # Add the main grid
-        self._mgr.AddPane(self.grid, wx.CENTER)
+        self._mgr.AddPane(self.grid, aui.AuiPaneInfo().
+                          Name("grid").Caption(_("Main grid")).CentrePane())
 
         # Tell the manager to 'commit' all the changes just made
         self._mgr.Update()
+        self._mgr.GetPane("attributes_toolbar")
+        self._mgr.Update()
 
         self._set_menu_toggles()
 
@@ -269,8 +269,7 @@ class MainWindow(wx.Frame, EventMixin):
                   handlers.OnFindToolbarToggle)
         self.Bind(self.EVT_CMD_ENTRYLINE_TOGGLE,
                   handlers.OnEntryLineToggle)
-        self.Bind(self.EVT_CMD_TABLECHOICE_TOGGLE,
-                  handlers.OnTableChoiceToggle)
+        self.Bind(aui.EVT_AUI_PANE_CLOSE, handlers.OnPaneClose)
 
         # File events
 
@@ -303,6 +302,7 @@ class MainWindow(wx.Frame, EventMixin):
         self.Bind(self.EVT_CMD_COPY, handlers.OnCopy)
         self.Bind(self.EVT_CMD_COPY_RESULT, handlers.OnCopyResult)
         self.Bind(self.EVT_CMD_PASTE, handlers.OnPaste)
+        self.Bind(self.EVT_CMD_PASTE_AS, handlers.OnPasteAs)
 
         # Help events
 
@@ -335,7 +335,7 @@ class MainWindow(wx.Frame, EventMixin):
 # End of class MainWindow
 
 
-class MainWindowEventHandlers(object):
+class MainWindowEventHandlers(EventMixin):
     """Contains main window event handlers"""
 
     def __init__(self, parent):
@@ -396,6 +396,8 @@ class MainWindowEventHandlers(object):
 
         self.main_window.grid.Refresh()
 
+        event.Skip()
+
     def OnSafeModeExit(self, event):
         """Safe mode exit event handler"""
 
@@ -409,6 +411,8 @@ class MainWindowEventHandlers(object):
 
         self.main_window.grid.Refresh()
 
+        event.Skip()
+
     def OnClose(self, event):
         """Program exit event handler"""
 
@@ -487,55 +491,72 @@ class MainWindowEventHandlers(object):
     def OnMainToolbarToggle(self, event):
         """Main window toolbar toggle event handler"""
 
-        main_toolbar = self.main_window._mgr.GetPane("main_window_toolbar")
+        self.main_window.main_toolbar.SetGripperVisible(True)
+        main_toolbar_info = \
+            self.main_window._mgr.GetPane("main_window_toolbar")
 
-        self._toggle_pane(main_toolbar)
+        self._toggle_pane(main_toolbar_info)
 
         event.Skip()
 
     def OnMacroToolbarToggle(self, event):
         """Macro toolbar toggle event handler"""
 
-        macro_toolbar = self.main_window._mgr.GetPane("macro_toolbar")
+        self.main_window.macro_toolbar.SetGripperVisible(True)
+        macro_toolbar_info = self.main_window._mgr.GetPane("macro_toolbar")
 
-        self._toggle_pane(macro_toolbar)
+        self._toggle_pane(macro_toolbar_info)
 
         event.Skip()
 
     def OnAttributesToolbarToggle(self, event):
         """Format toolbar toggle event handler"""
 
-        attributes_toolbar = \
+        self.main_window.attributes_toolbar.SetGripperVisible(True)
+        attributes_toolbar_info = \
             self.main_window._mgr.GetPane("attributes_toolbar")
 
-        self._toggle_pane(attributes_toolbar)
+        self._toggle_pane(attributes_toolbar_info)
 
         event.Skip()
 
     def OnFindToolbarToggle(self, event):
         """Search toolbar toggle event handler"""
 
-        find_toolbar = self.main_window._mgr.GetPane("find_toolbar")
+        self.main_window.find_toolbar.SetGripperVisible(True)
 
-        self._toggle_pane(find_toolbar)
+        find_toolbar_info = self.main_window._mgr.GetPane("find_toolbar")
+
+        self._toggle_pane(find_toolbar_info)
 
         event.Skip()
 
     def OnEntryLineToggle(self, event):
         """Entry line toggle event handler"""
 
-        entry_line = self.main_window._mgr.GetPane("entry_line")
+        entry_line_panel_info = \
+            self.main_window._mgr.GetPane("entry_line_panel")
 
-        self._toggle_pane(entry_line)
+        self._toggle_pane(entry_line_panel_info)
 
         event.Skip()
 
-    def OnTableChoiceToggle(self, event):
-        """Table choice toggle event handler"""
+    def OnPaneClose(self, event):
+        """Pane close toggle event handler (via close button)"""
 
-        table_choice = self.main_window._mgr.GetPane("table_choice")
+        toggle_label = event.GetPane().caption
 
-        self._toggle_pane(table_choice)
+        # Get menu item to toggle
+        menubar = self.main_window.menubar
+        toggle_id = menubar.FindMenuItem(_("View"), toggle_label)
+        toggle_item = menubar.FindItemById(toggle_id)
+
+        # Adjust toggle to pane visibility
+        toggle_item.Check(False)
+
+        menubar.UpdateMenus()
+
+        self.main_window._mgr.Update()
 
         event.Skip()
 
@@ -564,8 +585,6 @@ class MainWindowEventHandlers(object):
         if shape is None:
             return
 
-        self.main_window.grid.actions.change_grid_shape(shape)
-
         # Set new filepath and post it to the title bar
 
         self.main_window.filepath = None
@@ -583,16 +602,32 @@ class MainWindowEventHandlers(object):
         post_command_event(self.main_window, self.main_window.ResizeGridMsg,
                            shape=shape)
 
+        if is_gtk():
+            wx.Yield()
+
+        self.main_window.grid.actions.change_grid_shape(shape)
+
         self.main_window.grid.GetTable().ResetView()
         self.main_window.grid.ForceRefresh()
 
         # Display grid creation in status bar
-        statustext = _("New grid with dimensions {} created.").format(shape)
+        msg = _("New grid with dimensions {dim} created.").format(dim=shape)
         post_command_event(self.main_window, self.main_window.StatusBarMsg,
-                           text=statustext)
+                           text=msg)
 
         self.main_window.grid.ForceRefresh()
 
+        if is_gtk():
+            wx.Yield()
+
+        # Mark content as unchanged
+        try:
+            post_command_event(self.main_window, self.ContentChangedMsg,
+                               changed=False)
+        except TypeError:
+            # The main window does not exist any more
+            pass
+
     def OnOpen(self, event):
         """File open event handler"""
 
@@ -611,11 +646,14 @@ class MainWindowEventHandlers(object):
 
         # Get filepath from user
 
-        wildcard = _("Pyspread file (*.pys)|*.pys|All files (*.*)|*.*")
+        wildcard = \
+            _("Pyspread file") + " (*.pys)|*.pys|" + \
+            _("All files") + " (*.*)|*.*"
         message = _("Choose pyspread file to open.")
         style = wx.OPEN | wx.CHANGE_DIR
-        filepath, filterindex = self.interfaces.get_filepath_findex_from_user(
-                                    wildcard, message, style)
+        filepath, filterindex = \
+            self.interfaces.get_filepath_findex_from_user(wildcard, message,
+                                                          style)
 
         if filepath is None:
             return
@@ -637,6 +675,17 @@ class MainWindowEventHandlers(object):
 
         self.main_window.grid.ForceRefresh()
 
+        if is_gtk():
+            wx.Yield()
+
+        # Mark content as unchanged
+        try:
+            post_command_event(self.main_window, self.ContentChangedMsg,
+                               changed=False)
+        except TypeError:
+            # The main window does not exist any more
+            pass
+
     def OnSave(self, event):
         """File save event handler"""
 
@@ -665,11 +714,14 @@ class MainWindowEventHandlers(object):
 
         # Get filepath from user
 
-        wildcard = _("Pyspread file (*.pys)|*.pys|All files (*.*)|*.*")
+        wildcard = \
+            _("Pyspread file") + " (*.pys)|*.pys|" + \
+            _("All files") + " (*.*)|*.*"
         message = _("Choose filename for saving.")
         style = wx.SAVE | wx.CHANGE_DIR
-        filepath, filterindex = self.interfaces.get_filepath_findex_from_user(
-                                    wildcard, message, style)
+        filepath, filterindex = \
+            self.interfaces.get_filepath_findex_from_user(wildcard, message,
+                                                          style)
 
         if filepath is None:
             return 0
@@ -685,11 +737,13 @@ class MainWindowEventHandlers(object):
                 return 0
 
             # There is a file with the same path
-            message = _("The file {filepath} is already present.\nOverwrite?")\
-                                  .format(filepath=filepath)
-            short_message = _("File collision")
-            if not self.main_window.interfaces.get_warning_choice(
-                        message, short_message):
+            message = \
+                _("The file {filepath} is already present.\nOverwrite?")\
+                .format(filepath=filepath)
+            short_msg = _("File collision")
+
+            if not self.main_window.interfaces.get_warning_choice(message,
+                                                                  short_msg):
 
                 statustext = _("File present. Save aborted by user.")
                 post_command_event(self.main_window,
@@ -717,11 +771,14 @@ class MainWindowEventHandlers(object):
 
         # Get filepath from user
 
-        wildcard = _("Csv file (*.*)|*.*|Tab delimited text file (*.*)|*.*")
+        wildcard = \
+            _("CSV file") + " (*.*)|*.*|" + \
+            _("Tab delimited text file") + " (*.*)|*.*"
         message = _("Choose file to import.")
         style = wx.OPEN | wx.CHANGE_DIR
-        filepath, filterindex = self.interfaces.get_filepath_findex_from_user(
-                                    wildcard, message, style)
+        filepath, filterindex = \
+            self.interfaces.get_filepath_findex_from_user(wildcard, message,
+                                                          style)
 
         if filepath is None:
             return
@@ -748,38 +805,72 @@ class MainWindowEventHandlers(object):
 
         """
 
+        code_array = self.main_window.grid.code_array
+        tab = self.main_window.grid.current_table
+
         selection = self.main_window.grid.selection
 
         # Check if no selection is present
 
         selection_bbox = selection.get_bbox()
 
+        wildcard = _("CSV file") + " (*.*)|*.*"
+
         if selection_bbox is None:
-            # No selection --> Use current screen
+            # No selection --> Use smallest filled area for bottom right edge
+            maxrow, maxcol, __ = code_array.get_last_filled_cell(tab)
 
-            selection_bbox = self.main_window.grid.actions.get_visible_area()
+            (top, left), (bottom, right) = (0, 0), (maxrow, maxcol)
 
-        (top, left), (bottom, right) = selection_bbox
+        else:
+            (top, left), (bottom, right) = selection_bbox
 
         # Generator of row and column keys in correct order
 
-        code_array = self.main_window.grid.code_array
-        tab = self.main_window.grid.current_table
+        __top = 0 if top is None else top
+        __bottom = code_array.shape[0] if bottom is None else bottom + 1
+        __left = 0 if left is None else left
+        __right = code_array.shape[1] if right is None else right + 1
 
-        data = code_array[top:bottom + 1, left:right + 1, tab]
+        def data_gen(top, bottom, left, right):
+            for row in xrange(top, bottom):
+                yield (code_array[row, col, tab]
+                       for col in xrange(left, right))
+
+        data = data_gen(__top, __bottom, __left, __right)
+        preview_data = data_gen(__top, __bottom, __left, __right)
 
         # Get target filepath from user
 
-        wildcard = _("CSV file (*.*)|*.*")
+        # No selection --> Provide svg export of current cell
+        # if current cell is a matplotlib figure
+        if selection_bbox is None:
+            cursor = self.main_window.grid.actions.cursor
+            figure = code_array[cursor]
+            if isinstance(figure, Figure):
+                wildcard += \
+                    " |" + _("SVG file") + " (*.svg)|*.svg" + \
+                    " |" + _("EPS file") + " (*.eps)|*.eps" + \
+                    " |" + _("PS file") + " (*.ps)|*.ps" + \
+                    " |" + _("PDF file") + " (*.pdf)|*.pdf" + \
+                    " |" + _("PNG file") + " (*.png)|*.png"
+
         message = _("Choose filename for export.")
         style = wx.OPEN | wx.CHANGE_DIR
-        path, filterindex = self.interfaces.get_filepath_findex_from_user(
-                                    wildcard, message, style)
+        path, filterindex = \
+            self.interfaces.get_filepath_findex_from_user(wildcard, message,
+                                                          style)
+
+        # If an svg is exported then the selection bbox
+        # has to be changed to the current cell
+        if filterindex >= 1:
+            data = figure
 
         # Export file
         # -----------
 
-        self.main_window.actions.export_file(path, filterindex, data)
+        self.main_window.actions.export_file(path, filterindex, data,
+                                             preview_data)
 
     def OnApprove(self, event):
         """File approve event handler"""
@@ -788,14 +879,10 @@ class MainWindowEventHandlers(object):
             return
 
         msg = _(u"You are going to approve and trust a file that\n"
-                u"you have received from an untrusted source.\n"
-                u"After proceeding, the file is executed.\n"
-                u"It can harm your system as any program can.\n"
-                u"Unless you took precautions, it can delete your\n"
-                u"files or send them away over the Internet.\n"
-                u"CHECK EACH CELL BEFORE PROCEEDING.\n \n"
-                u"Do not forget cells outside the visible range.\n"
-                u"You have been warned.\n \n"
+                u"you have not created yourself.\n"
+                u"After proceeding, the file is executed.\n \n"
+                u"It may harm your system as any program can.\n"
+                u"Please check all cells thoroughly before\nproceeding.\n \n"
                 u"Proceed and sign this file as trusted?")
 
         short_msg = _("Security warning")
@@ -814,7 +901,7 @@ class MainWindowEventHandlers(object):
         """Clear globals event handler"""
 
         msg = _("Deleting globals and reloading modules cannot be undone."
-                    " Proceed?")
+                " Proceed?")
         short_msg = _("Really delete globals and modules?")
 
         choice = self.main_window.interfaces.get_warning_choice(msg, short_msg)
@@ -887,8 +974,8 @@ class MainWindowEventHandlers(object):
             focus.Copy()
 
         else:
-            data = self.main_window.actions.copy(
-                       self.main_window.grid.selection)
+            selection = self.main_window.grid.selection
+            data = self.main_window.actions.copy(selection)
             self.main_window.clipboard.set_clipboard(data)
 
         event.Skip()
@@ -896,8 +983,8 @@ class MainWindowEventHandlers(object):
     def OnCopyResult(self, event):
         """Clipboard copy results event handler"""
 
-        data = self.main_window.actions.copy_result(
-                            self.main_window.grid.selection)
+        selection = self.main_window.grid.selection
+        data = self.main_window.actions.copy_result(selection)
 
         # Check if result is a bitmap
         if type(data) is wx._gdi.Bitmap:
@@ -924,10 +1011,7 @@ class MainWindowEventHandlers(object):
 
         else:
             # We got a grid selection
-            grid = self.main_window.grid
-            key = (grid.GetGridCursorRow(),
-                   grid.GetGridCursorCol(),
-                   grid.current_table)
+            key = self.main_window.grid.actions.cursor
 
             self.main_window.actions.paste(key, data)
 
@@ -935,6 +1019,18 @@ class MainWindowEventHandlers(object):
 
         event.Skip()
 
+    def OnPasteAs(self, event):
+        """Clipboard paste as event handler"""
+
+        data = self.main_window.clipboard.get_clipboard()
+        key = self.main_window.grid.actions.cursor
+
+        self.main_window.actions.paste_as(key, data)
+
+        self.main_window.grid.ForceRefresh()
+
+        event.Skip()
+
     # View events
 
     def OnFocusFind(self, event):
@@ -951,8 +1047,9 @@ class MainWindowEventHandlers(object):
         cursor = self.main_window.grid.actions.cursor
         attr = self.main_window.grid.code_array.cell_attributes[cursor]
 
-        size, style, weight, font = [attr[name] for name in
-            ["pointsize", "fontstyle", "fontweight", "textfont"]]
+        size, style, weight, font = \
+            [attr[name] for name in ["pointsize", "fontstyle", "fontweight",
+                                     "textfont"]]
         current_font = wx.Font(int(size), -1, style, weight, 0, font)
 
         # Get Font from dialog
@@ -978,6 +1075,11 @@ class MainWindowEventHandlers(object):
                                self.main_window.FontItalicsMsg,
                                style=font.GetStyleString())
 
+            if is_gtk():
+                wx.Yield()
+
+            self.main_window.grid.update_attribute_toolbar()
+
     def OnTextColorDialog(self, event):
         """Event handler for launching text color dialog"""
 
@@ -1043,12 +1145,15 @@ class MainWindowEventHandlers(object):
 
         # Get filepath from user
 
-        wildcard = _("Macro file (*.py)|*.py|All files (*.*)|*.*")
+        wildcard = \
+            _("Macro file") + " (*.py)|*.py|" + \
+            _("All files") + " (*.*)|*.*"
         message = _("Choose macro file.")
 
         style = wx.OPEN | wx.CHANGE_DIR
-        filepath, filterindex = self.interfaces.get_filepath_findex_from_user(
-                                    wildcard, message, style)
+        filepath, filterindex = \
+            self.interfaces.get_filepath_findex_from_user(wildcard, message,
+                                                          style)
 
         if filepath is None:
             return
@@ -1068,12 +1173,15 @@ class MainWindowEventHandlers(object):
 
         # Get filepath from user
 
-        wildcard = _("Macro file (*.py)|*.py|All files (*.*)|*.*")
+        wildcard = \
+            _("Macro file") + " (*.py)|*.py|" + \
+            _("All files") + " (*.*)|*.*"
         message = _("Choose macro file.")
 
         style = wx.SAVE | wx.CHANGE_DIR
-        filepath, filterindex = self.interfaces.get_filepath_findex_from_user(
-                                    wildcard, message, style)
+        filepath, filterindex = \
+            self.interfaces.get_filepath_findex_from_user(wildcard, message,
+                                                          style)
 
         # Save macros to file
 
@@ -1088,7 +1196,7 @@ class MainWindowEventHandlers(object):
         """Manual launch event handler"""
 
         self.main_window.actions.launch_help("First steps in pyspread",
-            "First steps in pyspread.html")
+                                             "First steps.html")
 
     def OnTutorial(self, event):
         """Tutorial launch event handler"""
@@ -1105,7 +1213,7 @@ class MainWindowEventHandlers(object):
         """Python tutorial launch event handler"""
 
         self.main_window.actions.launch_help("Python tutorial",
-            "http://docs.python.org/tutorial/")
+                                             get_python_tutorial_path())
 
     def OnAbout(self, event):
         """About dialog event handler"""
