@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
-# Copyright 2011 Martin Manns
+# Copyright Martin Manns
 # Distributed under the terms of the GNU General Public License
 
 # --------------------------------------------------------------------
@@ -41,11 +41,14 @@ import datetime
 import os
 import types
 
+import wx
+
 from src.config import config
 
 from src.gui._events import post_command_event, StatusBarEventMixin
 
 import src.lib.i18n as i18n
+from src.lib.fileio import AOpen
 
 #use ugettext instead of getttext to avoid unicode errors
 _ = i18n.language.ugettext
@@ -59,9 +62,8 @@ def sniff(filepath):
 
     """
 
-    csvfile = open(filepath, "rb")
-    sample = csvfile.read(config["sniff_size"])
-    csvfile.close()
+    with open(filepath, "rb") as csvfile:
+        sample = csvfile.read(config["sniff_size"])
 
     sniffer = csv.Sniffer()
     dialect = sniffer.sniff(sample)()
@@ -73,17 +75,37 @@ def sniff(filepath):
 def get_first_line(filepath, dialect):
     """Returns List of first line items of file filepath"""
 
-    csvfile = open(filepath, "rb")
-    csvreader = csv.reader(csvfile, dialect=dialect)
+    with open(filepath, "rb") as csvfile:
+        csvreader = csv.reader(csvfile, dialect=dialect)
 
-    for first_line in csvreader:
-        break
-
-    csvfile.close()
+        for first_line in csvreader:
+            break
 
     return first_line
 
 
+def digested_line(line, digest_types):
+    """Returns list of digested values in line"""
+
+    digested_line = []
+    for i, ele in enumerate(line):
+        try:
+            digest_key = digest_types[i]
+
+        except IndexError:
+            digest_key = digest_types[0]
+
+        digest = Digest(acceptable_types=[digest_key])
+
+        try:
+            digested_line.append(repr(digest(ele)))
+
+        except Exception, err:
+            digested_line.append(str(err))
+
+    return digested_line
+
+
 def csv_digest_gen(filepath, dialect, has_header, digest_types):
     """Generator of digested values from csv file in filepath
 
@@ -98,34 +120,16 @@ def csv_digest_gen(filepath, dialect, has_header, digest_types):
 
     """
 
-    csvfile = open(filepath, "rb")
-    csvreader = csv.reader(csvfile, dialect=dialect)
+    with open(filepath, "rb") as csvfile:
+        csvreader = csv.reader(csvfile, dialect=dialect)
+
+        if has_header:
+            # Ignore first line
+            for line in csvreader:
+                break
 
-    if has_header:
-        # Ignore first line
         for line in csvreader:
-            break
-
-    for line in csvreader:
-        digested_line = []
-        for i, ele in enumerate(line):
-            try:
-                digest_key = digest_types[i]
-
-            except IndexError:
-                digest_key = digest_types[0]
-
-            digest = Digest(acceptable_types=[digest_key])
-
-            try:
-                digested_line.append(repr(digest(ele)))
-
-            except Exception, err:
-                digested_line.append(str(err))
-
-        yield digested_line
-
-    csvfile.close()
+            yield digested_line(line, digest_types)
 
 
 def cell_key_val_gen(iterable, shape, topleft=(0, 0)):
@@ -207,6 +211,10 @@ class Digest(object):
             if type(obj) is types.UnicodeType:
                 return obj
 
+            elif isinstance(obj, types.StringType):
+                # Try UTF-8
+                return obj.decode('utf-8')
+
             if obj is None:
                 return u""
 
@@ -261,8 +269,9 @@ class Digest(object):
         if self.fallback_type is not None and \
            self.fallback_type not in self.typehandlers:
 
-            err_msg = _("Fallback type {} unknown.").format(
-                                str(self.fallback_type))
+            err_msg = _("Fallback type {type} unknown.").\
+                format(type=str(self.fallback_type))
+
             raise NotImplementedError(err_msg)
 
     def __call__(self, orig_obj):
@@ -320,36 +329,23 @@ class CsvInterface(StatusBarEventMixin):
     def __iter__(self):
         """Generator of generators that yield csv data"""
 
-        try:
-            csv_file = open(self.path, "r")
+        with AOpen(self.path, "rb", main_window=self.main_window) as csv_file:
             csv_reader = csv.reader(csv_file, self.dialect)
 
-        except IOError, err:
-            statustext = "Error opening file " + self.path + "."
-            post_command_event(self.main_window, self.StatusBarMsg,
-                               text=statustext)
+            self.first_line = self.has_header
 
-            csv_file = []
-
-        self.first_line = self.has_header
-
-        try:
             for line in csv_reader:
                 yield self._get_csv_cells_gen(line)
-                self.first_line = False
+                break
 
-        except Exception, err:
-            msg = 'The file "' + self.csvfilename + '" only partly loaded.' + \
-                  '\n \nError message:\n' + str(err)
-            short_msg = 'Error reading CSV file'
-            self.main_window.interfaces.display_warning(msg, short_msg)
+            self.first_line = False
 
-        finally:
-            statustext = "File " + self.csvfilename + " imported successfully."
-            post_command_event(self.main_window, self.StatusBarMsg,
-                               text=statustext)
+            for line in csv_reader:
+                yield self._get_csv_cells_gen(line)
 
-        csv_file.close()
+        msg = _("File {filename} imported successfully.").format(
+            filename=self.csvfilename)
+        post_command_event(self.main_window, self.StatusBarMsg, text=msg)
 
     def _get_csv_cells_gen(self, line):
         """Generator of values in a csv line"""
@@ -359,7 +355,7 @@ class CsvInterface(StatusBarEventMixin):
         for j, value in enumerate(line):
             if self.first_line:
                 digest_key = None
-                digest = lambda x: x
+                digest = lambda x: repr(x)
             else:
                 try:
                     digest_key = digest_types[j]
@@ -385,13 +381,28 @@ class CsvInterface(StatusBarEventMixin):
     def write(self, iterable):
         """Writes values from iterable into CSV file"""
 
-        csvfile = open(self.path, "wb")
-        csv_writer = csv.writer(csvfile, self.dialect)
+        io_error_text = _("Error writing to file {filepath}.")
+        io_error_text = io_error_text.format(filepath=self.path)
 
-        for line in iterable:
-            csv_writer.writerow(line)
+        try:
 
-        csvfile.close()
+            with open(self.path, "wb") as csvfile:
+                csv_writer = csv.writer(csvfile, self.dialect)
+
+                for line in iterable:
+                    csv_writer.writerow(list(line))
+
+        except IOError:
+            txt = \
+                _("Error opening file {filepath}.").format(filepath=self.path)
+            try:
+                post_command_event(self.main_window, self.StatusBarMsg,
+                                   text=txt)
+            except TypeError:
+                # The main window does not exist any more
+                pass
+
+            return False
 
 
 class TxtGenerator(StatusBarEventMixin):
@@ -400,7 +411,7 @@ class TxtGenerator(StatusBarEventMixin):
     def __init__(self, main_window, path):
         self.main_window = main_window
         try:
-            self.infile = open(path, "r")
+            self.infile = open(path)
 
         except IOError:
             statustext = "Error opening file " + path + "."
@@ -417,4 +428,4 @@ class TxtGenerator(StatusBarEventMixin):
         for line in self.infile:
             yield (col for col in line.split())
 
-        self.infile.close()
\ No newline at end of file
+        self.infile.close()
